function [rom,abserr,hksv] = btsr(sys,r,tol); 
% BTSR
% Compute a reduced-order model of a given linear, continuous-time system
% using the SR implementation of balanced truncation based on spectral
% projection.
% 
% INPUT ARGUMENTS:
%   sys - an SS object containing the original system.  Note that sys.A is
%         assumed to be stable - this is not checked in the code!  An
%         unstable A-matrix will usually cause the sign function-based
%         Lyapunov solver to exit with an error due to failure to converge.
%    r  - desired order of the reduced-order model.  If r <= 0, a
%         reducded-order model is computed based on an absolute error less
%         than tol, where tol is either the 3rd input argument or set by 
%         default to 1e-4.  
%   tol - absolute error tolerance for the reduced-order model; used only if 
%         the 2nd input argument does not specify a desired order of the 
%         reduced-order model.  
%
% OUTPUT ARGUMENTS:
%   rom    - an SS object containing the reduced-order model. 
%   abserr - optional output argument, contains computed error bound for
%            reduced-order model.
%   hksv   - an optional output argument containing the computed Hankel
%            singular values.
%
% REFERENCE:
%
%   P. BENNER, E.S. QUINTANA-ORTI. Model reduction based on spectral 
%   projection methods.  In: P. Benner, V.L. Mehrmann, D.C. Sorensen
%   (eds.), "Dimension Redution of Large-Scale Systems", vol. 45 of
%   Lecture Notes in Computational Science and Engineering, pp. 5-48,
%   Springer-Verlag, Berlin/Heidelberg, 2005.
%
% Peter Benner,  2002-2005.

if sys.Ts ~= 0,  error('System must be continuous.'),  end
A = sys.A;
B = sys.B;
C = sys.C;

n = size(A,1);
m = size(B,2);
p = size(C,1);

tolselect = 0;
ordselect = 0;
if nargin > 1,
    if r <= 0,
        tolselect = 1;              
        if ( (nargin < 3) | ((nargin == 3) & (tol <= 0)) )
            tol = 1e-4;           
        end 
    else
        r         = floor(r);
        ordselect = 1;
    end
else
    tolselect = 1;
    tol       = 1e-4;
end
        
%% Compute full-rank factors of Gramians based on sign function method.
[S,R] = duallyap(A,B,C);

%% Uncomment to test accuracy of computed Gramians
%Wc = S*S';
%resS = norm(A*Wc+Wc*A'+B*B','fro')/(2*norm(A,'fro')*norm(Wc,'fro')+norm(B*B','fro'));
%disp(sprintf('Relatives Residuum in Wc = %12.4g', resS))
%Wo = R'*R;
%resR = norm(A'*Wo+Wo*A+C'*C,'fro')/(2*norm(A,'fro')*norm(Wo,'fro')+norm(C'*C,'fro'));
%disp(sprintf('Relatives Residuum in Wc = %12.4g', resR))

%% Compute Hankel singular values.
[U,Sigma,V]=svd(S'*R',0);
hksv = diag(Sigma);

%% Select order of reduced-order mdoel
s = min(size(S,2),size(R,1))
if tolselect,
    abserr = 0;
    r      = s+1;
    while abserr < tol/2,
        r      = r-1;
        abserr = abserr + hksv(r);
    end
else
    if r > s,
        warning('Desired size of reduced-order model too large.')
        disp(sprintf('         Set r = %i', s'))
        r = s;
    end
end

%% Compute reduced-order model.
S1  = diag(sqrt(hksv(1:r)));
U1  = U(:,1:r);
V1  = V(:,1:r);
Tl  = S1\V1'*R;
Tr  = S*U1/S1;
Ar  = Tl*A*Tr;
Br  = Tl*B;
Cr  = C*Tr;
rom = ss(Ar,Br,Cr,sys.D);
if nargout > 1,
    abserr = 2*(sum(hksv(r+1:s))+(n-s)*hksv(s));
end